@@ -2,7 +2,7 @@ package interp
 
 import (
 	"fmt"
-	"io/ioutil"
+	"io/fs"
 	"os"
 	"path/filepath"
 	"strings"
@@ -48,7 +48,7 @@ func (interp *Interpreter) importSrc(rPath, importPath string, skipTest bool) (s
 	}
 	interp.rdir[importPath] = true
 
-	files, err := ioutil.ReadDir(dir)
+	files, err := fs.ReadDir(interp.opt.filesystem, dir)
 	if err != nil {
 		return "", err
 	}
@@ -69,12 +69,20 @@ func (interp *Interpreter) importSrc(rPath, importPath string, skipTest bool) (s
 
 		name = filepath.Join(dir, name)
 		var buf []byte
-		if buf, err = ioutil.ReadFile(name); err != nil {
+		if buf, err = fs.ReadFile(interp.opt.filesystem, name); err != nil {
 			return "", err
 		}
 
+		n, err := interp.parse(string(buf), name, false)
+		if err != nil {
+			return "", err
+		}
+		if n == nil {
+			continue
+		}
+
 		var pname string
-		if pname, root, err = interp.ast(string(buf), name, false); err != nil {
+		if pname, root, err = interp.ast(n); err != nil {
 			return "", err
 		}
 		if root == nil {
@@ -97,7 +105,7 @@ func (interp *Interpreter) importSrc(rPath, importPath string, skipTest bool) (s
 
 		subRPath := effectivePkg(rPath, importPath)
 		var list []*node
-		list, err = interp.gta(root, subRPath, importPath)
+		list, err = interp.gta(root, subRPath, importPath, pkgName)
 		if err != nil {
 			return "", err
 		}
@@ -106,7 +114,7 @@ func (interp *Interpreter) importSrc(rPath, importPath string, skipTest bool) (s
 
 	// Revisit incomplete nodes where GTA could not complete.
 	for _, nodes := range revisit {
-		if err = interp.gtaRetry(nodes, importPath); err != nil {
+		if err = interp.gtaRetry(nodes, importPath, pkgName); err != nil {
 			return "", err
 		}
 	}
@@ -114,7 +122,7 @@ func (interp *Interpreter) importSrc(rPath, importPath string, skipTest bool) (s
 	// Generate control flow graphs.
 	for _, root := range rootNodes {
 		var nodes []*node
-		if nodes, err = interp.cfg(root, importPath); err != nil {
+		if nodes, err = interp.cfg(root, nil, importPath, pkgName); err != nil {
 			return "", err
 		}
 		initNodes = append(initNodes, nodes...)
@@ -185,13 +193,13 @@ func (interp *Interpreter) pkgDir(goPath string, root, importPath string) (strin
 	rPath := filepath.Join(root, "vendor")
 	dir := filepath.Join(goPath, "src", rPath, importPath)
 
-	if _, err := os.Stat(dir); err == nil {
+	if _, err := fs.Stat(interp.opt.filesystem, dir); err == nil {
 		return dir, rPath, nil // found!
 	}
 
 	dir = filepath.Join(goPath, "src", effectivePkg(root, importPath))
 
-	if _, err := os.Stat(dir); err == nil {
+	if _, err := fs.Stat(interp.opt.filesystem, dir); err == nil {
 		return dir, root, nil // found!
 	}
 
@@ -203,7 +211,7 @@ func (interp *Interpreter) pkgDir(goPath string, root, importPath string) (strin
 	}
 
 	rootPath := filepath.Join(goPath, "src", root)
-	prevRoot, err := previousRoot(rootPath, root)
+	prevRoot, err := previousRoot(interp.opt.filesystem, rootPath, root)
 	if err != nil {
 		return "", "", err
 	}
@@ -214,7 +222,7 @@ func (interp *Interpreter) pkgDir(goPath string, root, importPath string) (strin
 const vendor = "vendor"
 
 // Find the previous source root (vendor > vendor > ... > GOPATH).
-func previousRoot(rootPath, root string) (string, error) {
+func previousRoot(filesystem fs.FS, rootPath, root string) (string, error) {
 	rootPath = filepath.Clean(rootPath)
 	parent, final := filepath.Split(rootPath)
 	parent = filepath.Clean(parent)
@@ -227,7 +235,7 @@ func previousRoot(rootPath, root string) (string, error) {
 		// look for the closest vendor in one of our direct ancestors, as it takes priority.
 		var vendored string
 		for {
-			fi, err := os.Lstat(filepath.Join(parent, vendor))
+			fi, err := fs.Stat(filesystem, filepath.Join(parent, vendor))
 			if err == nil && fi.IsDir() {
 				vendored = strings.TrimPrefix(strings.TrimPrefix(parent, prefix), string(filepath.Separator))
 				break
